#!/bin/bash
# This is a helper script to auto-discover AWS configuration if not already passed in
# shellcheck disable=SC2154,SC1091
#

set -e
export AWS_METADATA_SERVICE_TIMEOUT=5
export AWS_METADATA_SERVICE_NUM_ATTEMPTS=3
env
print_header "Begin aws-discovery.sh"

if [ "${awsDeployment}" == "false" ] ; then
    cprint "Skipping aws-discovery.sh for non-AWS deployment."
    return
else
    cprint "AWS mode detected. Running aws discovery..."
fi

cprint "Checking for valid kube2iam aws credential..."
wait-until -t 120 -i 10 true curl --max-time 9 -f -s http://169.254.169.254/latest/meta-data/iam/security-credentials

if [ -z "${awsAccountNumber}" ] ; then
    cprint "global.awsAccountNumber not set, auto-discovering..."
    wait-until -t 30 -i 2 true curl --max-time 10 -f -s http://169.254.169.254/latest/dynamic/instance-identity/document
    accountNumber=$(curl --max-time 10 -s http://169.254.169.254/latest/dynamic/instance-identity/document | jq -r .accountId)
    export awsAccountNumber=${accountNumber}
else
    cprint "global.awsAccountNumber is set to ${awsAccountNumber}"
fi

if [ -z "${awsDefaultRegion}" ] ; then
    cprint "global.awsRegion not set, auto-discovering..."
    wait-until -t 30 -i 2 true curl --max-time 10 -f -s http://169.254.169.254/latest/meta-data/placement/region
    awsDefaultRegion=$(curl --max-time 10 -s http://169.254.169.254/latest/meta-data/placement/region)
    export AWS_DEFAULT_REGION=${awsDefaultRegion}
    echo "Setting awsRegion to: ${awsDefaultRegion}"
else
    cprint "global.awsRegion is set to ${awsDefaultRegion}"
    export AWS_DEFAULT_REGION=${awsDefaultRegion}
fi
export AWS_DEFAULT_REGION=${awsDefaultRegion}

if [ -z "${vpcId}" ] ; then
    cprint "global.awsVpcId not set, auto-discovering..."
    wait-until -t 30 -i 2 true curl --max-time 10 -f -s http://169.254.169.254/latest/meta-data/network/interfaces/macs/
    first_mac=$(curl -s http://169.254.169.254/latest/meta-data/network/interfaces/macs/ | head -1 | sed 's/\///')
    wait-until -t 30 -i 2 true curl --max-time 10 -f -s -L http://169.254.169.254/latest/meta-data/network/interfaces/macs/"${first_mac}"/vpc-id
    vpcId=$(curl --max-time 10 -s -L http://169.254.169.254/latest/meta-data/network/interfaces/macs/"${first_mac}"/vpc-id | head -1 )
    export TF_VAR_vpc_id=${vpcId}
else
    cprint "global.awsVpcId is set to ${vpcId}"
fi

if [ -z "${subnetIds}" ]; then
    cprint "global.awsSubnetIds not set, auto-discovering..."
    # aws someone return empty string. retry to work around the issue
    SECONDS=0
    while [ $SECONDS -le 30 ]; do
        data=$(aws ec2 describe-subnets --filter "Name=vpc-id,Values=${TF_VAR_vpc_id}" --query "Subnets[?MapPublicIpOnLaunch==\`false\`].SubnetId" --output text | xargs echo | sed 's/ /","/g')
        if [ "${data}x" !=  "x" ]; then break; fi
        sleep 2
        echo "$SECONDS secs retry subnet discovering.."
    done
    if [ "${data}x"  == x ]; then cprint "ERROR: failed to auto-discover subunets" 'red'; exit 1; fi
    subnetIds="[\"${data}\"]"
    export TF_VAR_subnet_ids=${subnetIds}
else
    cprint "global.awsSubnetIds is set to ${subnetIds}"
fi

if [ -z "${securityGroupIds}" ]; then
    cprint "global.awsSecurityGroupIds not set, auto-discovering..."
    # aws someone return empty string. retry to work around the issue
    SECONDS=0
    wait-until -t 30 -i 2 true curl --max-time 10 -f -s http://169.254.169.254/latest/meta-data/security-groups
    instanceSg=$(curl --max-time 10 -s http://169.254.169.254/latest/meta-data/security-groups)
    while [ $SECONDS -le 30 ]; do
        clusterName=$(aws ec2 describe-security-groups --filter "Name=group-name,Values=$instanceSg" --query "SecurityGroups[*].Tags[?Key=='aws:eks:cluster-name'].Value" --output text)
        cprint "EKS Cluster Name:${clusterName}"
        data=$(aws ec2 describe-security-groups --filter "Name=vpc-id,Values=${TF_VAR_vpc_id}" --query "SecurityGroups[?GroupName!='default']|[?contains(GroupName, '$clusterName') ==\`true\`].GroupId" --output text | xargs echo | sed 's/ /","/g')
        if [ "${data}x" !=  "x" ]; then break; fi
        sleep 2
        echo "$SECONDS secs retry security-groups discovering.."
    done
    if [ "${data}x"  == x ]; then cprint "ERROR: failed to auto-discover security-groups" 'red'; exit 1; fi
    securityGroupIds="[\"${data}\"]"
    export TF_VAR_security_group_ids=${securityGroupIds}
else
    cprint "global.securityGroupIds is set to ${securityGroupIds}"
fi

